"""
This module defines a class for a repository
"""
from candidate_repository import CandidateRepository
from interview_repository import InterviewRepository
from interviewround_repository import InterviewRoundRepository
from interviewer_repository import InterviewerRepository


class Repository:

    def __init__(self, data_storage):
        self.__data_storage = data_storage

        self.candidate_rep = CandidateRepository(self.__data_storage)

        self.interview_repository = InterviewRepository(self.__data_storage)

        self.interviewround_repository = InterviewRoundRepository(self.__data_storage)

        self.interviewer_repository = InterviewerRepository(self.__data_storage)



"""
This module defines an interviewer class
"""
from re import match

from bson import ObjectId


class Interviewer:

    def __init__(self, name, title="", email=""):

        self.__interviewround_ids = []

        self.___id = ObjectId()

        self.__name = name

        self.__title = title

        if match(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)", email.strip(), 0):
            self.__email = email

    def add_round(self, round_id):
        self.__interviewround_ids.append(round_id)

    def remove_round(self, round_id):
        self.__interviewround_ids.remove(round_id)

    @property
    def _id(self):
        return self.___id

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def title(self):
        return self.__title

    @title.setter
    def title(self, title):
        self.__title = title

    @property
    def email(self):
        return self.__email

    @email.setter
    def email(self, email):
        if match(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)", email.strip(), 0):
            self.__email = email

    @property
    def interviewround_ids(self):
        return self.__interviewround_ids



"""
This module defines a class for an interview round.
"""
import datetime as dt

from bson import ObjectId


class InterviewRound:

    def __init__(self, interview_id: ObjectId, datetime: dt.datetime = dt.datetime.now(), round_type: str = "",
                 round_format: str = "", status="Upcoming", mark: int = -1):

        self.__interviewer_ids = []

        self.___id = ObjectId()

        self.__interview_id = interview_id

        self.__datetime = datetime

        self.__round_type = round_type

        self.__round_format = round_format

        if status.strip().lower() == 'upcoming' or status.strip().lower() == 'ended':
            self.__status = status

        if 0 <= mark <= 100:
            self.__mark = mark

    def add_interviewer(self, interviewer_id):
        self.__interviewer_ids.append(interviewer_id)

    def remove_interviewer(self, interviewer_id):
        self.__interviewer_ids.remove(interviewer_id)

    @property
    def _id(self):
        return self.___id

    @property
    def interview_id(self):
        return self.__interview_id

    @interview_id.setter
    def interview_id(self, interview_id):
        self.__interview_id = interview_id

    @property
    def datetime(self):
        return self.__datetime

    @datetime.setter
    def datetime(self, datetime):
        self.__datetime = datetime

    @property
    def round_type(self):
        return self.__round_type

    @round_type.setter
    def round_type(self, round_type):
        self.__round_type = round_type

    @property
    def round_format(self):
        return self.__round_format

    @round_format.setter
    def round_format(self, round_format):
        self.__round_format = round_format

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, status):
        if status.strip().lower() == 'upcoming' or status.strip().lower() == 'ended':
            self.__status = status

    @property
    def mark(self):
        return self.__mark

    @mark.setter
    def mark(self, mark):
        self.__mark = mark

    @property
    def interview_ids(self):
        return self.__interviewer_ids



"""

"""


class CandidateRepository:

    def __init__(self, data_storage):
        self.__data_storage = data_storage

    def get_candidate_by_id(self, _id):
        return self.__data_storage.get_candidate_by_id(_id)

    def get_all_candidates(self):
        return self.__data_storage.get_all_candidates()

    def add_candidate(self, candidate):
        self.__data_storage.add_candidate(candidate)

    def update_candidate(self, candidate):
        self.__data_storage.update_candidate(candidate)

    def remove_candidate(self, _id):
        self.__data_storage.remove_candidate(_id)



"""
This module defines a class for interview
"""
import datetime as dt

from bson import ObjectId


class Interview:

    def __init__(self, candidate_id: ObjectId, start_time: dt.date = dt.date.today(), decision: str = "In progress",
                 status: str = "Active"):

        self.___id = ObjectId()

        self.__candidate_id = candidate_id

        self.__start_time = start_time

        if (decision.lower().strip() == "in progress" or decision.lower().strip() == "hire"
                or decision.lower().strip() == "reject"):
            self.__decision = decision

        if status.lower().strip() == "active" or status.lower().strip() == "archived":
            self.__status = status

        self.__mark: int = -1

        self.__end_time: dt.date = dt.date.today() + dt.timedelta(days=14)

    @property
    def _id(self):
        return self.___id

    @property
    def candidate_id(self):
        return self.__candidate_id

    @candidate_id.setter
    def candidate_id(self, candidate_id):
        self.__candidate_id = candidate_id

    @property
    def start_time(self):
        return self.__start_time

    @start_time.setter
    def start_time(self, start_time):
        self.__start_time = start_time

    @property
    def decision(self):
        return self.__decision

    @decision.setter
    def decision(self, decision):
        if (decision.lower().strip() == "in progress" or decision.lower().strip() == "reject"
                or decision.lower().strip() == "hire"):
            self.__decision = decision

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, status):
        if status.lower().strip() == "active" or status.lower().strip() == "archived":
            self.__status = status

    @property
    def mark(self):
        return self.__mark

    @mark.setter
    def mark(self, mark):
        if 0 <= mark <= 100:
            self.__mark = mark

    @property
    def end_time(self):
        return self.__end_time

    @end_time.setter
    def end_time(self, end_time):
        self.__end_time = end_time



"""

"""


class InterviewerRepository:

    def __init__(self, data_storage):
        self.__data_storage = data_storage

    def get_interviewer(self, _id):
        return self.__data_storage.get_interviewer(_id)

    def get_interviewers_for_round(self, round_id):
        return self.__data_storage.get_interviewers_for_round(round_id)

    def get_all_interviewers(self):
        return self.__data_storage.get_all_interviewers()

    def add_interviewer(self, interviewer):
        self.__data_storage.add_interviewer(interviewer)

    def remove_interviewer(self, _id):
        self.__data_storage.remove(_id)

    def update_interviewer(self, interviewer):
        self.__data_storage.update(interviewer)



"""
This module defines a class for a candidate
"""
from re import match
import datetime as dt

from bson import ObjectId


class Candidate:

    def __init__(self, name: str, role: str = "", yoe: int = 0, email: str = "", phone_number: str = "",
                 linkedin: str = "", birthdate: dt.date = dt.date.today(), is_blacklisted: bool = False,
                 notes: str = ""):

        self.___id = ObjectId()

        self.__name = name

        self.__role = role

        if 0 <= yoe <= 50:
            self.__yoe = yoe

        if match(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)", email.strip(), 0):
            self.__email = email

        if match(r'\+?\d+', phone_number, flags=0):
            self.__phone_number = phone_number

        self.__linkedin = linkedin

        self.__birthdate = birthdate

        self.__is_blacklisted = is_blacklisted

        self.__notes = notes

    @property
    def _id(self):
        return self.___id

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def role(self):
        return self.__role

    @role.setter
    def role(self, role):
        self.__role = role

    @property
    def yoe(self):
        return self.__yoe

    @yoe.setter
    def yoe(self, yoe):
        if 0 <= yoe <= 50:
            self.__yoe = yoe

    @property
    def email(self):
        return self.__email

    @email.setter
    def email(self, email):
        if match(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)", email.strip(), 0):
            self.__email = email

    @property
    def phone_number(self):
        return self.__phone_number

    @phone_number.setter
    def phone_number(self, phone_number):
        if match(r'\+?\d+', phone_number, flags=0):
            self.__phone_number = phone_number

    @property
    def linkedin(self):
        return self.__linkedin

    @linkedin.setter
    def linkedin(self, linkedin):
        self.__linkedin = linkedin

    @property
    def birthdate(self):
        return self.__birthdate

    @birthdate.setter
    def birthdate(self, birthdate):
        self.__birthdate = birthdate

    @property
    def is_blacklisted(self):
        return self.__is_blacklisted

    @is_blacklisted.setter
    def is_blacklisted(self, is_blacklisted):
        self.__is_blacklisted = is_blacklisted

    @property
    def notes(self):
        return self.__notes

    @notes.setter
    def notes(self, notes):
        self.__notes = notes



"""

"""


class InterviewRepository:

    def __init__(self, data_storage):
        self.__data_storage = data_storage

    def get_interviews_for_candidate(self, cand_id):
        return self.__data_storage.get_interviews_for_candidate(cand_id)

    def get_interview_by_id(self, _id):
        return self.__data_storage.get_interview_by_id(_id)

    def add_interview(self, interview):
        self.__data_storage.add_interview(interview)

    def remove_interview(self, _id):
        self.__data_storage.remove_interview(_id)

    def update_interview(self, _id, **fields):
        self.__data_storage.update_interview(_id, fields)

    def get_mark(self, _id):
        self.__data_storage.get_mark(_id)



from setuptools import setup, find_packages

VERSION = '0.0.1'
DESCRIPTION = 'My first Python package'
LONG_DESCRIPTION = 'My first Python package with a slightly longer description'

# Setting up
setup(
    # the name must match the folder name 'verysimplemodule'
    name="hr_tool_logic",
    version=VERSION,
    author="Ну я",
    author_email="<youremail@email.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(include=['hr_tool_logic']),
)


"""

"""


class InterviewRoundRepository:

    def __init__(self, data_storage):

        self.__data_storage = data_storage

    def get_rounds_for_interview(self, _id):

        return self.__data_storage.get_rounds_for_interview(_id)

    def get_round_by_id(self, _id):

        return self.__data_storage.get_round_by_id(_id)

    def add_round(self, interview_round):

        self.__data_storage.add_round(interview_round)

    def remove_round(self, _id):

        self.__data_storage.remove(_id)

    def update_round(self, interviewround):

        self.__data_storage.update_round(interviewround)
